#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <avr/pgmspace.h>
#include "avr_compat.h"
#include "main.h"
#include "oled.h"
#include "usart.h"


void init ();
void led ( char val );
void disp_vcc_en ( char val );
void set_relay ( char relay, char val );
void set_fan ( char val );
void set_glow ( char val );
void set_pump ( char val );
void timerinit();
void status ( char *text );
unsigned short read_TEMP ();
inline void setTERMO_CS(int v);
void printTemp ( char *text );
void print_terminal ( char *text );
void init_terminal ();


volatile unsigned char taste1=0, taste2=0, taste3=0, mode=0, currentmode=100, flamme=0, temptrigger=1, serialtrigger=1;
volatile unsigned int sekunde=0, statetimeout=0;
volatile unsigned short temp;

char terminal[4][20];

SIGNAL (SIG_OVERFLOW0) {

	TCNT0=255-30;// calibration of 1000ints / sec.

	// if (PORTD & (1<<LED)) led (1); else led(0);
	
	if (!(BUTTONPIN & (1<<BUTTON0))) taste1++;
	if (!(BUTTONPIN & (1<<BUTTON1))) taste2++;
	if (!(BUTTONPIN & (1<<BUTTON2))) taste3++;

	if (!(FLAMMEPIN & (1<<FLAMME))) {
		if (flamme<255) flamme++; }
	else  {
		if (flamme>0) flamme--;
	}

	sekunde++;
	if (sekunde>=1000) { 
		sekunde=0;
		temptrigger=1;
		serialtrigger=1;
		if (statetimeout) statetimeout--;

		if (PORTD & (1<<LED)) led (1); else led(0);
	}
}

/*
	MODES (mode)

	0	off/standy
	1	lften (motorlanlauf)
	2	vorglhen (glhkerze aufwrmen)
	3	diesel ein, warte auf zndung (temp, flammsensor)
	4	betrieb, vorglhen aus (berwachung der temp)
	5	nachlauf (temp, flammensor, timeout)

	6   heizen - temp watchdog aktiv.


*/


int main ()
{
	init();
	led (1); // enable led
	disp_vcc_en (1); // enable 12V vcc for display.
	
	USART_Init(51); // 9600baud
	SendString("Serial Port open...\n\r");
	
	display_init();
	lcd_puts(0,0, "Hello (cold) World...");
	
	timerinit();
	init_terminal();
	read_TEMP (); // bulk read to initialize sensor

		

	/* 
	_delay_ms(1000);
	set_fan (1);
	set_glow (1);
	set_pump (1);
	*/

	while (1) {
		
		if (taste1>2) {
			// lcd_puts((8*1)+1,0, "1...");
			taste1=0;
			if (mode==0) { mode = 1; init_terminal(); }

		}
		
		if (taste2>2) {
			if (mode==5) {
				print_terminal("Neustart...");
				mode = 2;
			}
			taste2=0;
		}
	
		if (taste3>2) {
			// lcd_puts((8*3)+1,0, "3...");
			taste3=0;
			if (mode != 0) mode=5;
		}


		if (temptrigger) {
			temp = read_TEMP ();
	        if (temp & (1<<2)) {
	            // pc.printf("Thermocouple input open! \n\r");
	            printTemp  ("TC. nicht verb.!");
			
	            temp=0;
	        } else { 
	            temp = (temp>>5);
	            char display[20];
				sprintf(display, "AIR OUT TEMP: %d C", temp);
				printTemp (display);
	            // pc.printf("$1;1;0;%d;0\r\n", temp/4);

	        }
			temptrigger=0;
		}	
	
		if (serialtrigger) {
			// do update message for remote in and check for input data
		}
	
	
	switch (mode) {
		case 0:
			if (currentmode != mode) {  // user just switched!
				currentmode = mode;
				set_fan(0);
				set_pump(0);
				set_fan(0);
				status ("IDLE...");
				print_terminal("taste fuer start!");
			}

		break;
		case 1:
			if (currentmode != mode) {  // user just switched!
					status ("Motorhochlauf...");
					print_terminal("--------------------");
					set_fan (1);
					currentmode = mode;
					statetimeout=5;
			}
			if (statetimeout<=0) mode=2;
		break;
		case 2:
			if (currentmode != mode) {  // user just switched!
				status ("vorwaermen...");
				set_fan (1);
				set_glow (1);
				currentmode = mode;
				statetimeout=10;
			}
			if (statetimeout<=0) mode=3;
		break;
		case 3:
			if (currentmode != mode) {  // user just switched!
				status ("warte auf zuendung...");
				set_pump (1);
				currentmode = mode;
				statetimeout=90;
				flamme=0;
			}
			if (statetimeout<=0) {
				// mode=3;
				// timeout due preglow (keine zndung)
				// status ("zuendung timeout.");
				print_terminal("zuendung timeout.");
				print_terminal("gluhkerze prufen!");
				set_pump(0);
				set_fan(0);
				set_glow(0);
				mode=0;
			}
			if (flamme>100) { // flammsensor hat brennvorgang erkannt (evtl. noch auf thermoelement umrsten)
				//flamme contact is closed for min 100ms
				mode=4;
				char display[20];
				sprintf(display, "heizbeginn: %d C", temp);
				print_terminal(display);
			}
		break;
		case 4:
			if (currentmode != mode) {  // user just switched!
				set_glow(0);
				status ("Heizen...");
				currentmode = mode;
			}
			if (flamme<=0) {
				print_terminal("Das Feuer ist aus");
				mode=5;
			}
			if (temp>100) {
				mode=6;
			}
		break;
		case 5:
			if (currentmode != mode) {  // user just switched!
				status ("Nachlauf...");
				currentmode = mode;
				statetimeout=(10*60);
				set_pump(0);
				set_glow(0);
			}
			if ((flamme<=0)&&(temp<40)) {
				mode=0;
				print_terminal("Nachlauf beendet.");
				char display[20];
				sprintf(display, "rest temp: %d C", temp);
				print_terminal(display);
				
				mode=0;
			}
			if (statetimeout<=0) {
				print_terminal("Nachlauf timeout.");
				mode=0;
			}
		break;
		case 6:
			if (currentmode != mode) {  // user just switched!
				set_glow(0);
				status ("Watchdog aktiv.");
				currentmode = mode;
			}
			if (flamme<=0) {
				print_terminal("Das Feuer ist aus");
				mode=5;
			}
			if (temp<90) {
				print_terminal("Temperaturabfall.");
				print_terminal("Abschaltung...");
				mode=5;
			}
		break;
		}
	
	
	}

}


void init ()
{
	// configure IO ports here...
	
	sbi (DDRD, LED);
	led (0); // disbale led

	sbi (DDRC, RELAY0);
	sbi (DDRC, RELAY1);
	sbi (DDRC, RELAY2);

	disp_vcc_en (0);
	sbi (DDRB, PB0); // enable 12v control.

	sbi (DDRB, DISP_RESET);
	sbi (DDRB, DISP_DC);
	sbi (DDRB, DISP_CS);

	sbi (DDRD, THERMO_CS);

}

void led ( char val )
{
	if (val) cbi (LEDPORT, LED); else sbi (LEDPORT, LED);
}

void disp_vcc_en ( char val )
{
	if (val) sbi (PORTB, PB0); else cbi (PORTB, PB0);
}

void set_relay ( char relay, char val )
{
	switch (relay)
	{
		case 0:
			if (val) sbi (PORTC, RELAY0); else cbi (PORTC, RELAY0);
		break;
		case 1:
			if (val) sbi (PORTC, RELAY1); else cbi (PORTC, RELAY1);
		break;
		case 2:
			if (val) sbi (PORTC, RELAY2); else cbi (PORTC, RELAY2);
		break;
	}
}

void set_fan ( char val )
{
	set_relay (0,val);
}

void set_glow ( char val )
{
	set_relay (1,val);
}

void set_pump ( char val )
{
	set_relay (2,val);
}

void timerinit()
{
	 sbi (TCCR0, 2); // prescaler clk/256

	sbi (TIMSK, 0); // overflow int. enable
	sei();  
}

void status ( char *text ) 
{
	
	clear_lines ((8*7)+1, 7);

	lcd_puts((8*7)+1,0, text);

}

void printTemp ( char *text ) 
{
	
	clear_lines ((8*6)+1, 7);

	lcd_puts((8*6)+1,0, text);

}

unsigned short read_TEMP ()
{
    unsigned short temp = 0;
    
	sbi (SPCR, CPHA); // sample data on falling clock edge! 
    setTERMO_CS(0);

	
	temp |= (out8(0)<<8);
	temp |= out8(0);

    setTERMO_CS(1);

	cbi (SPCR, CPHA); // switch back to make display working again. ;)



    return temp;
}

inline void setTERMO_CS(int v)
{
    if (v) sbi (PORTD, THERMO_CS); else cbi (PORTD, THERMO_CS);
}

void print_terminal ( char *text )
{
	for (int i=0; i<4;i++)
		memcpy(terminal[i], terminal[i+1], 20);
	
	memset(terminal[4], 0, 20);
	strcpy(terminal[4], text);
	
	for (int i=0; i<=4;i++)
		lcd_putn((8*(i+1))+1,0, terminal[i], 20);

	// _delay_ms(1000);

}

void init_terminal ()
{
	memset(terminal, 0, 5*20);
	clear_lines ((8*1)+1, (8*5));
}

