#include <string.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "oled.h"
#include "lcdfont.h"
#include "avr_compat.h"


inline void setRESET(int v)
{
	if (v)
		PORTB |= (1<<PB2);
	else
		PORTB &=~(1<<PB2);
}

inline void setCS(int v)
{
	if (v)
		PORTB |= (1<<PB3);
	else
		PORTB &=~(1<<PB3);
}

inline void setDC(int v)
{
	if (v)
		PORTB |= (1<<PB1);
	else
		PORTB &=~(1<<PB1);
}

unsigned char out8(unsigned char x)
{
	SPDR = x;
	// sputs_P(PSTR("."));	
	while (!(SPSR & (1<<SPIF)));
	return SPDR;
}

void send_command(char *x, int len)
{
	// sputs_P(PSTR("Send Command!!eins11\r\n"));	
	setDC(0);
	setCS(0);
	while (len--)
		out8(*x++);
	setCS(1);
}

void send_data(char *x, int len)
{
	setDC(1);
	setCS(0);
	while (len--)
		out8(*x++);
	setDC(0);
	out8(0xE3); // NOP
	setCS(1);
}

void lcd_puts(int y, int x, const char *text)
{
		int line;
		char col[] = {0x15, x/2, (x + strlen(text)*6)/2 - 1};
		char row[] = {0x75, y, y + 7};
	  send_command(col, 3);
	  send_command(row, 3);

	  for (line = 0; line < 8; ++line)
	  {
	  	int i;
	  	for (i=0; i<strlen(text); ++i)
	  	{
		  	char data[3] = {0,0,0};
		  	int ax;
		  	for (ax = 0; ax < 6; ++ax)
		  	{
		  		int v = pgm_read_byte((lcdfont + text[i] * 6 + ax)) & (1<<line);
		  		if (ax & 1)
			  		data[ax/2] |= v ? 0xF : 0;
			  	else
			  		data[ax/2] |= v ? 0xF0 : 0;
		  	
		  	}
		  	// data[0]=0xff;
			send_data(data, 3);
		  }
	  }
}

void lcd_putn(int y, int x, const char *text, int n)
{
		int line;
		char col[] = {0x15, x/2, (x + n *6)/2 - 1};
		char row[] = {0x75, y, y + 7};
	  send_command(col, 3);
	  send_command(row, 3);

	  for (line = 0; line < 8; ++line)
	  {
	  	int i;
	  	for (i=0; i < n; ++i)
	  	{
		  	char data[3] = {0,0,0};
		  	int ax;
		  	for (ax = 0; ax < 6; ++ax)
		  	{
		  		int v = pgm_read_byte((lcdfont + text[i] * 6 + ax)) & (1<<line);
		  		if (ax & 1)
			  		data[ax/2] |= v ? 0xF : 0;
			  	else
			  		data[ax/2] |= v ? 0xF0 : 0;
		  	
		  	}
		  	// data[0]=0xff;
			send_data(data, 3);
		  }
	  }
}

void cls() 
{
	send_command("\x15\x00\x3f", 3); // set column address
	send_command("\x75\x00\x3f", 3); // set row address
	int x, y;
	for (x=0;x<64;x++)
		for (y=0;y<64;y++)
			send_data("\x00", 1);

}

void clear_lines (char a, char b)
{

	char row[] = {0x75, a, a + 7};
	send_command(row, 3);
	send_command("\x15\x00\x3f", 3); // set column address

	int x, y;
	for (x=0;x<=b;x++)
		for (y=0;y<64;y++)
			send_data("\x00", 1);
}

void display_init ()
{
	
	SPIDDR |= SCLK |MOSI;
	CSDDR |= CS;
	SPIPORT &= ~(SCLK|MOSI|MISO);
	_delay_us(400);
	CSPORT |= CS;

	SPCR = (1<<SPE) | (1<<MSTR); //   | (1<<SPR1)  | (1<<SPR0);    /* enable SPI as master, set clk divider */
	// SPSR = 1; // double speed mode!!! (kicks ass!!)
	

	
	_delay_us(400);
	
	setRESET(0);
	setCS(1);
	setRESET(1);	

	send_command("\x15\x00\x3f", 3); // set column address
	send_command("\x75\x00\x3f", 3); // set row address
	send_command("\x81\x5f", 2); // set KONTRAST
//	send_command("\x81\x01", 2); // set KONTRAST
	send_command("\x86",1); // burn!!! (full current range)
	send_command("\xa0\x52", 2); // remap
	send_command("\xa1\x00", 2); // set display start line
	send_command("\xa2\x4c", 2); // set display offset
	send_command("\xa8\x3f", 2); // set multiplex
	send_command("\xaf", 1); // set display ON
	send_command("\xa4", 1); // set display mode
	send_command("\xad\x02", 2); // disable DC-DC converter
	send_command("\xcf\xf0", 2); // dc dc bias current
	send_command("\xb2\x46", 2); // row period
	send_command("\xb0\x08", 2); // precharge comp enable
	send_command("\xb4\x00", 2); // precharge comp. level
	send_command("\xb3\x41", 2); // set clock divide
	send_command("\xb1\x22", 2); // phase length
	send_command("\xbf\x0d", 2); // set vsl
	send_command("\xbe\x00", 2); // set vcomh
	send_command("\xbc\x10", 2); // set vprecharge
	send_command("\xb8\x01\x01\x10\x02\0x20\x02\x30\x03\x40\x04\x50\x05\0x60\x06\x70", 16);

	cls ();
}
